@@ -10,11 +10,13 @@ import ctypes
 import logging
 import sys
 import time
+from typing import Optional, Union
 
-from can import BusABC, Message
-from can.util import time_perfcounter_correlation
+from can import BitTiming, BitTimingFd, BusABC, CanProtocol, Message
+from can.exceptions import CanError, CanInitializationError, CanOperationError
+from can.typechecking import CanFilters
+from can.util import check_or_adjust_timing_clock, time_perfcounter_correlation
 
-from ...exceptions import CanError, CanInitializationError, CanOperationError
 from . import constants as canstat
 from . import structures
 
@@ -62,7 +64,6 @@ def __get_canlib_function(func_name, argtypes=None, restype=None, errcheck=None)
 
 
 class CANLIBError(CanError):
-
     """
     Try to display errors that occur within the wrapped C library nicely.
     """
@@ -199,6 +200,17 @@ if __canlib is not None:
         errcheck=__check_status_initialization,
     )
 
+    canSetBusParamsC200 = __get_canlib_function(
+        "canSetBusParamsC200",
+        argtypes=[
+            c_canHandle,
+            ctypes.c_byte,
+            ctypes.c_byte,
+        ],
+        restype=canstat.c_canStatus,
+        errcheck=__check_status_initialization,
+    )
+
     canSetBusParamsFd = __get_canlib_function(
         "canSetBusParamsFd",
         argtypes=[
@@ -360,7 +372,13 @@ class KvaserBus(BusABC):
     The CAN Bus implemented for the Kvaser interface.
     """
 
-    def __init__(self, channel, can_filters=None, **kwargs):
+    def __init__(
+        self,
+        channel: int,
+        can_filters: Optional[CanFilters] = None,
+        timing: Optional[Union[BitTiming, BitTimingFd]] = None,
+        **kwargs,
+    ):
         """
         :param int channel:
             The Channel id to create this bus with.
@@ -370,6 +388,12 @@ class KvaserBus(BusABC):
 
         Backend Configuration
 
+        :param timing:
+            An instance of :class:`~can.BitTiming` or :class:`~can.BitTimingFd`
+            to specify the bit timing parameters for the Kvaser interface. If provided, it
+            takes precedence over the all other timing-related parameters.
+            Note that the `f_clock` property of the `timing` instance must be 16_000_000 (16MHz)
+            for standard CAN or 80_000_000 (80MHz) for CAN FD.
         :param int bitrate:
             Bitrate of channel in bit/s
         :param bool accept_virtual:
@@ -404,10 +428,19 @@ class KvaserBus(BusABC):
             computer, set this to True or set single_handle to True.
         :param bool fd:
             If CAN-FD frames should be supported.
+        :param bool fd_non_iso:
+            Open the channel in Non-ISO (Bosch) FD mode. Only applies for FD buses.
+            This changes the handling of the stuff-bit counter and the CRC. Defaults
+            to False (ISO mode)
+        :param bool exclusive:
+            Don't allow sharing of this CANlib channel.
+        :param bool override_exclusive:
+            Open the channel even if it is opened for exclusive access already.
         :param int data_bitrate:
             Which bitrate to use for data phase in CAN FD.
             Defaults to arbitration bitrate.
-
+        :param bool no_init_access:
+            Don't open the handle with init access.
         """
 
         log.info(f"CAN Filters: {can_filters}")
@@ -420,34 +453,53 @@ class KvaserBus(BusABC):
         driver_mode = kwargs.get("driver_mode", DRIVER_MODE_NORMAL)
         single_handle = kwargs.get("single_handle", False)
         receive_own_messages = kwargs.get("receive_own_messages", False)
+        exclusive = kwargs.get("exclusive", False)
+        override_exclusive = kwargs.get("override_exclusive", False)
         accept_virtual = kwargs.get("accept_virtual", True)
-        fd = kwargs.get("fd", False)
+        no_init_access = kwargs.get("no_init_access", False)
+        fd = isinstance(timing, BitTimingFd) if timing else kwargs.get("fd", False)
         data_bitrate = kwargs.get("data_bitrate", None)
+        fd_non_iso = kwargs.get("fd_non_iso", False)
 
         try:
             channel = int(channel)
         except ValueError:
-            raise ValueError("channel must be an integer")
+            raise ValueError("channel must be an integer") from None
+
         self.channel = channel
+        self.single_handle = single_handle
+        self._can_protocol = CanProtocol.CAN_20
+        if fd_non_iso:
+            self._can_protocol = CanProtocol.CAN_FD_NON_ISO
+        elif fd:
+            self._can_protocol = CanProtocol.CAN_FD
 
         log.debug("Initialising bus instance")
-        self.single_handle = single_handle
-
         num_channels = ctypes.c_int(0)
         canGetNumberOfChannels(ctypes.byref(num_channels))
         num_channels = int(num_channels.value)
         log.info("Found %d available channels", num_channels)
         for idx in range(num_channels):
             channel_info = get_channel_info(idx)
+            channel_info = f'{channel_info["device_name"]}, S/N {channel_info["serial"]} (#{channel_info["dongle_channel"]})'
             log.info("%d: %s", idx, channel_info)
             if idx == channel:
                 self.channel_info = channel_info
 
         flags = 0
+        if exclusive:
+            flags |= canstat.canOPEN_EXCLUSIVE
+        if override_exclusive:
+            flags |= canstat.canOPEN_OVERRIDE_EXCLUSIVE
         if accept_virtual:
             flags |= canstat.canOPEN_ACCEPT_VIRTUAL
+        if no_init_access:
+            flags |= canstat.canOPEN_NO_INIT_ACCESS
         if fd:
-            flags |= canstat.canOPEN_CAN_FD
+            if fd_non_iso:
+                flags |= canstat.canOPEN_CAN_FD_NONISO
+            else:
+                flags |= canstat.canOPEN_CAN_FD
 
         log.debug("Creating read handle to bus channel: %s", channel)
         self._read_handle = canOpenChannel(channel, flags)
@@ -457,22 +509,43 @@ class KvaserBus(BusABC):
             ctypes.byref(ctypes.c_long(TIMESTAMP_RESOLUTION)),
             4,
         )
-
-        if fd:
-            if "tseg1" not in kwargs and bitrate in BITRATE_FD:
-                # Use predefined bitrate for arbitration
-                bitrate = BITRATE_FD[bitrate]
-            if data_bitrate in BITRATE_FD:
-                # Use predefined bitrate for data
-                data_bitrate = BITRATE_FD[data_bitrate]
-            elif not data_bitrate:
-                # Use same bitrate for arbitration and data phase
-                data_bitrate = bitrate
-            canSetBusParamsFd(self._read_handle, data_bitrate, tseg1, tseg2, sjw)
+        if isinstance(timing, BitTimingFd):
+            timing = check_or_adjust_timing_clock(timing, [80_000_000])
+            canSetBusParams(
+                self._read_handle,
+                timing.nom_bitrate,
+                timing.nom_tseg1,
+                timing.nom_tseg2,
+                timing.nom_sjw,
+                1,
+                0,
+            )
+            canSetBusParamsFd(
+                self._read_handle,
+                timing.data_bitrate,
+                timing.data_tseg1,
+                timing.data_tseg2,
+                timing.data_sjw,
+            )
+        elif isinstance(timing, BitTiming):
+            timing = check_or_adjust_timing_clock(timing, [16_000_000])
+            canSetBusParamsC200(self._read_handle, timing.btr0, timing.btr1)
         else:
-            if "tseg1" not in kwargs and bitrate in BITRATE_OBJS:
-                bitrate = BITRATE_OBJS[bitrate]
-        canSetBusParams(self._read_handle, bitrate, tseg1, tseg2, sjw, no_samp, 0)
+            if fd:
+                if "tseg1" not in kwargs and bitrate in BITRATE_FD:
+                    # Use predefined bitrate for arbitration
+                    bitrate = BITRATE_FD[bitrate]
+                if data_bitrate in BITRATE_FD:
+                    # Use predefined bitrate for data
+                    data_bitrate = BITRATE_FD[data_bitrate]
+                elif not data_bitrate:
+                    # Use same bitrate for arbitration and data phase
+                    data_bitrate = bitrate
+                canSetBusParamsFd(self._read_handle, data_bitrate, tseg1, tseg2, sjw)
+            else:
+                if "tseg1" not in kwargs and bitrate in BITRATE_OBJS:
+                    bitrate = BITRATE_OBJS[bitrate]
+            canSetBusParams(self._read_handle, bitrate, tseg1, tseg2, sjw, no_samp, 0)
 
         # By default, use local echo if single handle is used (see #160)
         local_echo = single_handle or receive_own_messages
@@ -485,13 +558,26 @@ class KvaserBus(BusABC):
             1,
         )
 
+        # enable canMSG_LOCAL_TXACK flag in received messages
+
+        canIoCtlInit(
+            self._read_handle,
+            canstat.canIOCTL_SET_LOCAL_TXACK,
+            ctypes.byref(ctypes.c_byte(local_echo)),
+            1,
+        )
+
         if self.single_handle:
             log.debug("We don't require separate handles to the bus")
             self._write_handle = self._read_handle
         else:
             log.debug("Creating separate handle for TX on channel: %s", channel)
-            self._write_handle = canOpenChannel(channel, flags)
-            canBusOn(self._read_handle)
+            if exclusive:
+                flags_ = flags & ~canstat.canOPEN_EXCLUSIVE
+                flags_ |= canstat.canOPEN_OVERRIDE_EXCLUSIVE
+            else:
+                flags_ = flags
+            self._write_handle = canOpenChannel(channel, flags_)
 
         can_driver_mode = (
             canstat.canDRIVER_SILENT
@@ -499,9 +585,24 @@ class KvaserBus(BusABC):
             else canstat.canDRIVER_NORMAL
         )
         canSetBusOutputControl(self._write_handle, can_driver_mode)
-        log.debug("Going bus on TX handle")
+
+        self._is_filtered = False
+        super().__init__(
+            channel=channel,
+            can_filters=can_filters,
+            **kwargs,
+        )
+
+        # activate channel after CAN filters were applied
+        log.debug("Go on bus")
+        if not self.single_handle:
+            canBusOn(self._read_handle)
         canBusOn(self._write_handle)
 
+        # timestamp must be set after bus is online, otherwise kvReadTimer may return erroneous values
+        self._timestamp_offset = self._update_timestamp_offset()
+
+    def _update_timestamp_offset(self) -> float:
         timer = ctypes.c_uint(0)
         try:
             if time.get_clock_info("time").resolution > 1e-5:
@@ -509,18 +610,15 @@ class KvaserBus(BusABC):
                 kvReadTimer(self._read_handle, ctypes.byref(timer))
                 current_perfcounter = time.perf_counter()
                 now = ts + (current_perfcounter - perfcounter)
-                self._timestamp_offset = now - (timer.value * TIMESTAMP_FACTOR)
+                return now - (timer.value * TIMESTAMP_FACTOR)
             else:
                 kvReadTimer(self._read_handle, ctypes.byref(timer))
-                self._timestamp_offset = time.time() - (timer.value * TIMESTAMP_FACTOR)
+                return time.time() - (timer.value * TIMESTAMP_FACTOR)
 
         except Exception as exc:
             # timer is usually close to 0
             log.info(str(exc))
-            self._timestamp_offset = time.time() - (timer.value * TIMESTAMP_FACTOR)
-
-        self._is_filtered = False
-        super().__init__(channel=channel, can_filters=can_filters, **kwargs)
+            return time.time() - (timer.value * TIMESTAMP_FACTOR)
 
     def _apply_filters(self, filters):
         if filters and len(filters) == 1:
@@ -545,7 +643,7 @@ class KvaserBus(BusABC):
                     for extended in (0, 1):
                         canSetAcceptanceFilter(handle, 0, 0, extended)
             except (NotImplementedError, CANLIBError) as e:
-                log.error("An error occured while disabling filtering: %s", e)
+                log.error("An error occurred while disabling filtering: %s", e)
 
     def flush_tx_buffer(self):
         """Wipeout the transmit buffer on the Kvaser."""
@@ -586,6 +684,7 @@ class KvaserBus(BusABC):
             is_remote_frame = bool(flags & canstat.canMSG_RTR)
             is_error_frame = bool(flags & canstat.canMSG_ERROR_FRAME)
             is_fd = bool(flags & canstat.canFDMSG_FDF)
+            is_rx = not bool(flags & canstat.canMSG_LOCAL_TXACK)
             bitrate_switch = bool(flags & canstat.canFDMSG_BRS)
             error_state_indicator = bool(flags & canstat.canFDMSG_ESI)
             msg_timestamp = timestamp.value * TIMESTAMP_FACTOR
@@ -597,6 +696,7 @@ class KvaserBus(BusABC):
                 is_error_frame=is_error_frame,
                 is_remote_frame=is_remote_frame,
                 is_fd=is_fd,
+                is_rx=is_rx,
                 bitrate_switch=bitrate_switch,
                 error_state_indicator=error_state_indicator,
                 channel=self.channel,
@@ -686,16 +786,19 @@ class KvaserBus(BusABC):
 
     @staticmethod
     def _detect_available_configs():
-        num_channels = ctypes.c_int(0)
+        config_list = []
+
         try:
+            num_channels = ctypes.c_int(0)
             canGetNumberOfChannels(ctypes.byref(num_channels))
+
+            for channel in range(0, int(num_channels.value)):
+                info = get_channel_info(channel)
+
+                config_list.append({"interface": "kvaser", "channel": channel, **info})
         except (CANLIBError, NameError):
             pass
-
-        return [
-            {"interface": "kvaser", "channel": channel}
-            for channel in range(num_channels.value)
-        ]
+        return config_list
 
 
 def get_channel_info(channel):
@@ -722,7 +825,11 @@ def get_channel_info(channel):
         ctypes.sizeof(number),
     )
 
-    return f"{name.value.decode('ascii')}, S/N {serial.value} (#{number.value + 1})"
+    return {
+        "device_name": name.value.decode("ascii", errors="replace"),
+        "serial": serial.value,
+        "dongle_channel": number.value + 1,
+    }
 
 
 init_kvaser_library()
