@@ -27,17 +27,14 @@ import os
 import struct
 import sys
 import time
-from typing import Dict, List, Tuple, Union
 
 from can import __version__
-
-from .logger import (
-    _append_filter_argument,
-    _create_base_argument_parser,
-    _create_bus,
-    _parse_additional_config,
-    _parse_filters,
+from can.cli import (
+    _set_logging_level_from_namespace,
+    add_bus_arguments,
+    create_bus_from_namespace,
 )
+from can.typechecking import TDataStructs
 
 logger = logging.getLogger("can.viewer")
 
@@ -162,14 +159,13 @@ class CanViewer:  # pylint: disable=too-many-instance-attributes
 
     # Unpack the data and then convert it into SI-units
     @staticmethod
-    def unpack_data(cmd: int, cmd_to_struct: Dict, data: bytes) -> List[float]:
+    def unpack_data(cmd: int, cmd_to_struct: TDataStructs, data: bytes) -> list[float]:
         if not cmd_to_struct or not data:
             # These messages do not contain a data package
             return []
 
-        for key in cmd_to_struct:
+        for key, value in cmd_to_struct.items():
             if cmd == key if isinstance(key, int) else cmd in key:
-                value = cmd_to_struct[key]
                 if isinstance(value, tuple):
                     # The struct is given as the fist argument
                     struct_t: struct.Struct = value[0]
@@ -391,7 +387,9 @@ class SmartFormatter(argparse.HelpFormatter):
             return super()._fill_text(text, width, indent)
 
 
-def parse_args(args: List[str]) -> Tuple:
+def _parse_viewer_args(
+    args: list[str],
+) -> tuple[argparse.Namespace, TDataStructs]:
     # Parse command line arguments
     parser = argparse.ArgumentParser(
         "python -m can.viewer",
@@ -412,9 +410,8 @@ def parse_args(args: List[str]) -> Tuple:
         allow_abbrev=False,
     )
 
-    # Generate the standard arguments:
-    # Channel, bitrate, data_bitrate, interface, app_name, CAN-FD support
-    _create_base_argument_parser(parser)
+    # add bus options group
+    add_bus_arguments(parser, filter_arg=True, group_title="Bus arguments")
 
     optional = parser.add_argument_group("Optional arguments")
 
@@ -471,8 +468,6 @@ def parse_args(args: List[str]) -> Tuple:
         default="",
     )
 
-    _append_filter_argument(optional, "-f")
-
     optional.add_argument(
         "-v",
         action="count",
@@ -488,8 +483,8 @@ def parse_args(args: List[str]) -> Tuple:
         raise SystemExit(errno.EINVAL)
 
     parsed_args, unknown_args = parser.parse_known_args(args)
-
-    can_filters = _parse_filters(parsed_args)
+    if unknown_args:
+        print("Unknown arguments:", unknown_args)
 
     # Dictionary used to convert between Python values and C structs represented as Python strings.
     # If the value is 'None' then the message does not contain any data package.
@@ -511,9 +506,7 @@ def parse_args(args: List[str]) -> Tuple:
     # similarly the values
     # are divided by the value in order to convert from real units to raw integer values.
 
-    data_structs: Dict[
-        Union[int, Tuple[int, ...]], Union[struct.Struct, Tuple, None]
-    ] = {}
+    data_structs: TDataStructs = {}
     if parsed_args.decode:
         if os.path.isfile(parsed_args.decode[0]):
             with open(parsed_args.decode[0], encoding="utf-8") as f:
@@ -528,7 +521,7 @@ def parse_args(args: List[str]) -> Tuple:
             key, fmt = int(tmp[0], base=16), tmp[1]
 
             # The scaling
-            scaling: List[float] = []
+            scaling: list[float] = []
             for t in tmp[2:]:
                 # First try to convert to int, if that fails, then convert to a float
                 try:
@@ -541,19 +534,13 @@ def parse_args(args: List[str]) -> Tuple:
             else:
                 data_structs[key] = struct.Struct(fmt)
 
-    additional_config = _parse_additional_config(
-        [*parsed_args.extra_args, *unknown_args]
-    )
-    return parsed_args, can_filters, data_structs, additional_config
+    return parsed_args, data_structs
 
 
 def main() -> None:
-    parsed_args, can_filters, data_structs, additional_config = parse_args(sys.argv[1:])
-
-    if can_filters:
-        additional_config.update({"can_filters": can_filters})
-    bus = _create_bus(parsed_args, **additional_config)
-
+    parsed_args, data_structs = _parse_viewer_args(sys.argv[1:])
+    bus = create_bus_from_namespace(parsed_args)
+    _set_logging_level_from_namespace(parsed_args)
     curses.wrapper(CanViewer, bus, data_structs)  # type: ignore[attr-defined,unused-ignore]
 
 
